unit TimedTicket;

interface

uses
  SysUtils, Classes, Controls, Forms, StdCtrls, Buttons, IniFiles,
  Ticket;

type
	TOpenTime = record
   	Hour: Longint;
    Min: Integer;
    Sec: Integer;
  end;
  TFormTimedTicket = class(TFormTicket)
    LabelOpenTime: TLabel;
    BtnCount: TSpeedButton;
    BtnClear: TSpeedButton;
    BtnStop: TSpeedButton;
    procedure BtnCountClick(Sender: TObject);
    procedure BtnClearClick(Sender: TObject);
    procedure BtnStopClick(Sender: TObject);
    procedure LabelOpenTimeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
  	FCountEn: Boolean;
  	FOnCountEn: TNotifyEvent;
    OpenTime: TOpenTime;
		procedure ShowSts(En: Boolean);
    procedure SetCountEn(En: Boolean);
  protected
    procedure HandleTick; override;
    procedure SaveImage(TicketImage: TStringList); override;
  public
  	property CountEn: Boolean read FCountEn write SetCountEn;
    property OnCountEn: TNotifyEvent read FOnCountEn write FOnCountEn;
    procedure Save(       IniFile: TIniFile;
    							 const	Section: string); override;
    procedure Restore(      IniFile: TIniFile;
    							 		const	Section: string); override;
    procedure Assign(Source: TPersistent); override;
  end;

implementation

{$R *.DFM}

uses
	Dialogs, Graphics,
	Util, Main;

const
	TimeSpace = '          ';
	TimeFormat = '%d:%2.2d:%2.2d';

function AddTime(T1,T2: TOpenTime): TOpenTime;
begin

  Result.Sec := T1.Sec + T2.Sec;
  Result.Min := T1.Min + T2.Min;
  Result.Hour := T1.Hour + T2.Hour;
	if Result.Sec >= 0 then begin
		Result.Min := Result.Min + (Result.Sec div 60);
	  Result.Sec := Result.Sec mod 60;
  end
  else begin
  	while Result.Sec < 0 do begin
    	Dec(Result.Min);
      Result.Sec := Result.Sec + 60;
    end;
  end;
	if Result.Min >= 0 then begin
		Result.Hour := Result.Hour + (Result.Min div 60);
  	Result.Min := Result.Min mod 60;
  end
  else begin
  	while Result.Min < 0 do begin
    	Dec(Result.Hour);
      Result.Min := Result.Min + 60;
    end;
  end;

end;

procedure TFormTimedTicket.ShowSts(En: Boolean);
begin

	if En then begin
	  LabelOpenTime.Color := clRed;
    Icon := FormMain.ResourceIcon['ACTIVETICKETICON'];
  end
  else begin
	  LabelOpenTime.Color := clYellow;
    Icon := FormMain.ResourceIcon['TICKETICON'];
  end;

end;

procedure TFormTimedTicket.SetCountEn(En: Boolean);
begin

	FCountEn := En;
  ShowSts(FCountEn);
	BtnCount.Down := En;
  TickEnabled := En;

end;

procedure TFormTimedTicket.HandleTick;
begin

	with OpenTime do begin
  	Inc(Sec);
    if Sec >= 60 then begin
			Sec := 0;
    	Inc(Min);
      if Min >= 60 then begin
      	Min := 0;
        Inc(Hour);
      end;
    end;
	  LabelOpenTime.Caption := Format(TimeSpace + TimeFormat,[Hour,Min,Sec]);
  end;

end;

procedure TFormTimedTicket.Save(				IniFile: TIniFile;
    							 							const 	Section: string);
begin

	inherited Save(IniFile,Section);
	with IniFile,OpenTime do
  	WriteString(Section,'OpenTime',Format(TimeFormat,[Hour,Min,Sec]));

end;

procedure TFormTimedTicket.Restore(				IniFile: TIniFile;
 							 										 const	Section: string);

var
	S: string;

begin

	inherited Restore(IniFile,Section);
	with IniFile,OpenTime do begin
	  S := ReadString(Section,'OpenTime','');
    if S = '' then begin
    	Hour := 0;
      Min := 0;
      Sec := 0;
    end
    else begin
	    try
				Hour := StrToInt(ParseStr(S,':'));
				Min := StrToInt(ParseStr(S,':'));
				Sec := StrToInt(S);
	    except
  	  	Hour := 0;
    	  Min := 0;
      	Sec := 0;
	    end;
    end;
 	  LabelOpenTime.Caption := Format(TimeSpace + TimeFormat,[Hour,Min,Sec]);
	end;

end;

procedure TFormTimedTicket.Assign(Source: TPersistent);
begin

  inherited Assign(Source);
	with Source as TFormTimedTicket do begin
    Self.OpenTime := OpenTime;
    Self.LabelOpenTime.Caption := LabelOpenTime.Caption;
    Self.OnCountEn := OnCountEn;
  end;

end;

procedure TFormTimedTicket.SaveImage(TicketImage: TStringList);
begin

  inherited SaveImage(TicketImage);
	with TicketImage,OpenTime do
    Insert(1,'Timed for: ' +  Format(TimeFormat,[Hour,Min,Sec]));

end;

procedure TFormTimedTicket.BtnCountClick(Sender: TObject);
begin

	if not FCountEn then begin
	 	FCountEn := true;
  	ShowSts(true);
 		FormMain.SetTray('ACTIVETICKETICON',Caption);
    TickEnabled := true;
 		if Assigned(FOnCountEn) then FOnCountEn(Self);
  end;

end;

procedure TFormTimedTicket.BtnStopClick(Sender: TObject);
begin

	if FCountEn then begin
    TickEnabled := false;
	 	FCountEn := false;
  	ShowSts(false);
 		FormMain.SetTray('TICKETICON',Application.Title);
  end;

end;

procedure TFormTimedTicket.BtnClearClick(Sender: TObject);
begin

	with OpenTime do begin
    Memo.Lines.Insert(0,FormatDateTime('dd/mm/yyyy hh:nn - ',Now) + Format(TimeFormat,[Hour,Min,Sec]));
  	Hour := 0;
    Min := 0;
    Sec := 0;
 	  LabelOpenTime.Caption := Format(TimeSpace + TimeFormat,[Hour,Min,Sec]);
  end;

end;

procedure TFormTimedTicket.LabelOpenTimeClick(Sender: TObject);

var
	S: string;
  TimeDelta,NewTime: TOpenTime;

begin

  with TimeDelta do begin
	  S := '00:00';
    Hour := 0;
    Min := 0;
    Sec := 0;
    if InputQuery('Add Time','',S) then begin
    	try
        Hour := StrToInt(ParseStr(S,':'));
        if Length(S) > 0 then
					Min := StrToInt(ParseStr(S,':'));
        if Length(S) > 0 then
					Sec := StrToInt(S);
        NewTime := AddTime(OpenTime,TimeDelta);
        S := Format('New Open Time is ' + TimeFormat,[NewTime.Hour,NewTime.Min,NewTime.Sec]);
      	if MessageDlg(S,mtConfirmation,mbOkCancel,0) = mrOk then begin
        	OpenTime := AddTime(OpenTime,TimeDelta);
			 	  LabelOpenTime.Caption := Format(TimeSpace + TimeFormat,[OpenTime.Hour,OpenTime.Min,OpenTime.Sec]);
        end;
      except
      	MessageDlg('Invalid Time',mtWarning,[mbOk],0);
      end;
    end;
  end;

end;

procedure TFormTimedTicket.FormCreate(Sender: TObject);
begin

  inherited;
	FCountEn := false;
  BtnCount.Down := false;
  LabelOpenTime.Color := clYellow;
  LabelOpenTime.Caption := Format(TimeSpace + TimeFormat,[0,0,0]);

end;

initialization

	TFormTimedTicket.Register;

end.

